Package: ModalForecast
Title: Parametric Modal ARIMA Models using the SKD Family
Version: 0.1.0
Authors@R: person("Christian", "Galarza", email = "chedgala@espol.edu.ec", role = c("aut", "cre"))
Description: Implements parametric modal Autoregressive Integrated Moving Average (ARIMA) models utilizing the Skewed Distribution (SKD) family. Current distributions supported are the Skew-Normal, Skewed Student-t, and Skewed Laplace. The conditional mode is parameterized and optimized via maximum likelihood using analytical gradients. Includes comprehensive residual diagnostics, robustness options (heavy tails, asymmetry), robust parametric bootstrap prediction intervals, and classical asymptotic inference via the Fisher Information matrix. Methods are described in Galarza, C.E., Lachos, V.H., Cabral, C.R.B., & Castro, L.M. (2017) <doi:10.1002/sta4.140>.
URL: https://github.com/chedgala/ModalForecast
BugReports: https://github.com/chedgala/ModalForecast/issues
Depends: R (>= 3.5.0)
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: stats, graphics, forecast, ggplot2, gridExtra, scales, grid
Suggests: rmarkdown, testthat (>= 3.0.0), knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-06 15:02:39 UTC; chedgala
Author: Christian Galarza [aut, cre]
Maintainer: Christian Galarza <chedgala@espol.edu.ec>
Repository: CRAN
Date/Publication: 2026-05-12 17:40:07 UTC
Built: R 4.5.2; ; 2026-05-12 17:50:36 UTC; unix

Package: ModalForecast
Title: Parametric Modal ARIMA Models using the SKD Family
Version: 0.1.0
Authors@R: person("Christian", "Galarza", email = "chedgala@espol.edu.ec", role = c("aut", "cre"))
Description: Implements parametric modal Autoregressive Integrated Moving Average (ARIMA) models utilizing the Skewed Distribution (SKD) family. Current distributions supported are the Skew-Normal, Skewed Student-t, and Skewed Laplace. The conditional mode is parameterized and optimized via maximum likelihood using analytical gradients. Includes comprehensive residual diagnostics, robustness options (heavy tails, asymmetry), robust parametric bootstrap prediction intervals, and classical asymptotic inference via the Fisher Information matrix. Methods are described in Galarza, C.E., Lachos, V.H., Cabral, C.R.B., & Castro, L.M. (2017) <doi:10.1002/sta4.140>.
URL: https://github.com/chedgala/ModalForecast
BugReports: https://github.com/chedgala/ModalForecast/issues
Depends: R (>= 3.5.0)
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: stats, graphics, forecast, ggplot2, gridExtra, scales, grid
Suggests: rmarkdown, testthat (>= 3.0.0), knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-06 15:02:39 UTC; chedgala
Author: Christian Galarza [aut, cre]
Maintainer: Christian Galarza <chedgala@espol.edu.ec>
Repository: CRAN
Date/Publication: 2026-05-12 17:40:07 UTC
Built: R 4.5.2; ; 2026-05-12 19:26:39 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(AIC,modal_arima)
S3method(BIC,modal_arima)
S3method(coef,modal_arima)
S3method(diagnostics,modal_arima)
S3method(envelope,modal_arima)
S3method(fitted,modal_arima)
S3method(forecast,modal_arima)
S3method(logLik,modal_arima)
S3method(plot,modal_arima)
S3method(predict,modal_arima)
S3method(print,modal_arima)
S3method(residuals,modal_arima)
S3method(summary,modal_arima)
export(auto.modal.arima)
export(diagnostics)
export(envelope)
export(fit_modal_arima)
export(forecast)
import(ggplot2)
importFrom(forecast,forecast)
importFrom(forecast,ndiffs)
importFrom(graphics,plot)
importFrom(grid,gpar)
importFrom(grid,textGrob)
importFrom(gridExtra,grid.arrange)
importFrom(scales,comma_format)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,Box.test)
importFrom(stats,acf)
importFrom(stats,arima)
importFrom(stats,coef)
importFrom(stats,dnorm)
importFrom(stats,frequency)
importFrom(stats,integrate)
importFrom(stats,is.ts)
importFrom(stats,logLik)
importFrom(stats,optim)
importFrom(stats,pacf)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,qnorm)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,residuals)
importFrom(stats,rgamma)
importFrom(stats,runif)
importFrom(stats,shapiro.test)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,ts)
importFrom(stats,uniroot)

# ModalForecast

<!-- badges: start -->
<!-- badges: end -->

The `ModalForecast` package implements parametric modal ARIMA models utilizing the Skewed Distribution (SKD) family. Instead of connecting the expected value (mean) to covariates, the model connects the **conditional mode** to the systematic autoregressive integrated moving average (ARIMA) components. 

By modeling the mode directly, this framework helps mitigate the effects of localized extremes, asymmetry, and non-normal behavior, providing robust centralized predictions under asymmetric error distributions.

## Methodology

### The Skewed Distribution (SKD) Family
To construct a modal regression model, we require a flexible parametric continuous distribution where the mode is explicitly parameterized and differentiable. We adopt the generalized SKD family, which supports robust inference through heavy tails and asymmetry. The package currently implements the **Skew-Normal**, **Skewed Student-t**, and **Skewed Laplace** distributions.

> **Note on the "normal" distribution:** In `ModalForecast`, specifying `dist = "normal"` does not invoke the standard symmetric Gaussian distribution. Instead, it refers to the **Skew-Normal** distribution from the SKD family. The standard normal is recovered asymptotically only when the estimated skewness parameter $\gamma = 1$.

Let $y_t \in \mathbb{R}$ be the response variable at time $t$. We assume $y_t$ follows a distribution from the SKD family with mode $\mu_t$, scale $\sigma$, skewness parameter $\gamma \in (0,\infty)$ (or effectively $p \in (0,1)$), and tail parameter(s) $\boldsymbol{\nu}$:

$$y_t \sim \text{SKD}(\mu_t, \sigma, \gamma, \boldsymbol{\nu})$$

The family is constructed using a scale mixture representation that ensures mathematical tractability while allowing heavy tails. A crucial property of this parameterization is that the probability density function reaches its global maximum exactly at $y_t = \mu_t$. Consequently, $\mu_t$ represents the true conditional mode of the distribution.

### Systematic Component: Modal ARIMA

Instead of the standard mean-based ARIMA, we model the sequence of conditional modes $\mu_t$:

$$ \mu_t = c + \sum_{i=1}^p \phi_i y_{t-i} + \sum_{j=1}^q \theta_j \epsilon_{t-j} $$

where $\epsilon_t = y_t - \mu_t$ is the asymmetric prediction error, $p$ is the autoregressive order, and $q$ is the moving average order. 

The parameter vector $\boldsymbol{\Theta} = (c, \boldsymbol{\phi}, \boldsymbol{\theta}, \sigma, \gamma, \boldsymbol{\nu})^\top$ is estimated via Maximum Likelihood Estimation (MLE). Conditionally on the initial values, the log-likelihood function over the sequence of $n$ observations is constructed explicitly as a function of $y_t$:

$$ \ell(\boldsymbol{\Theta}) = \sum_{t=1}^n \log f(y_t | \mu_t, \sigma, \gamma, \boldsymbol{\nu}) $$

where $f(\cdot)$ is the probability density function of the chosen SKD distribution (e.g., Skew-Normal, Skewed Student-t, Skewed Laplace), and $\mu_t$ embeds the recursive ARIMA structure.
## Installation

You can install the development version of ModalForecast from GitHub with:

```r
# install.packages("devtools")
devtools::install_github("chedgala/ModalForecast")
```

## Example Application: Lynx Dataset

The following plots demonstrate the diagnostic capabilities and forecasting performance of the `ModalForecast` package using the well-known `lynx` dataset.

### Diagnostic Envelopes and Inference
The `envelope()` function constructs simulation envelopes based on the exact theoretical distance distributions of the SKD family (e.g., half-normal, half-t, exponential). This provides a visually intuitive goodness-of-fit assessment to help select the best distribution among `normal`, `t`, or `laplace`. 

Below is an evaluation of the Skew-Normal, Skewed Student-t, and Skewed Laplace fits on the `lynx` dataset:

<p align="center">
  <img src="man/figures/diagnostics_lynx_premium.png" width="80%">
</p>

The `diagnostics()` function provides additional analysis including ACF/PACF of Randomized Quantile Residuals (RQR), and the `summary()` method handles analytical Fisher Information matrix standard errors.

### Out-of-Sample Forecasting
Comparison between traditional Gaussian ARIMA (Mean) and the Modal ARIMA (Mode) utilizing different members of the SKD family. The package computes both **Asymptotic** prediction intervals for standard series, and **Parametric Bootstrap** simulated prediction intervals for greater coverage in small sample settings.

<p align="center">
  <img src="man/figures/forecast_lynx_premium.png" width="80%">
</p>

## Quick Start Tutorial

Below is a brief tutorial showing how to adjust a modal ARIMA model to empirical data.

```r
library(ModalForecast)
library(forecast)

# 1. Load Empirical Data (Lynx)
data(lynx)
y <- log10(lynx) # log-transformation is common for this dataset

# 2. Find the best SKD Error Distribution (Normal vs T vs Laplace) 
# We fit a base Model and compare Information Criteria (e.g. AIC)
fit_n <- fit_modal_arima(y, order=c(2, 0, 0), dist="normal")
fit_t <- fit_modal_arima(y, order=c(2, 0, 0), dist="t")
fit_l <- fit_modal_arima(y, order=c(2, 0, 0), dist="laplace")

c(Normal = AIC(fit_n), Student = AIC(fit_t), Laplace = AIC(fit_l))

# 3. Use the rigorous Auto Modal ARIMA selector globally on the best distribution
# Since Skew-Normal returned the lowest AIC (-4.46), it wins. 
fit_auto <- auto.modal.arima(y, d=0, max.p=5, max.q=5, dist="normal")

# 4. Print the automatically fitted modal model
print(fit_auto)

# Model Summary & Diagnostics
summary(fit_auto)
diagnostics(fit_auto)

# Forecast modal trajectory with multiple confidence levels (alphas)
pred <- forecast(fit_auto, h=10, level=c(80, 95, 99))

# The package natively supports the 'forecast' library ecosystem:
library(forecast)

# 1. Plot rich visualization with confidence bands identical to standard ARIMA
autoplot(pred)

# 2. Evaluate forecast metric diagnostic functions (ME, RMSE, MAE, MAPE, etc.)
accuracy(pred)
```

The fitted object returns standard coefficients, scale `sigma`, and skewness `gamma`. If `gamma` is significantly different from 1, it indicates positive asymmetry (right-skewness if $>1$) or negative asymmetry ($<1$), capturing patterns that typical least-squares ARIMA would miss. If the specified distribution possesses additional tail parameters (like `nu` for Skewed Student-t), they are estimated automatically.

## References

* Galarza, C. E., Lachos, V. H., Cabral, C. R. B., & Castro, L. M. (2017). Robust quantile regression using a generalized class of skewed distributions. *Stat*, 6(1), 113-130. https://doi.org/10.1002/sta4.140

# Testing scripts
library(osiris); library(dplyr);library(tibble);library(ncdf4);library(rlang); library(data.table)

# Test for Step 0: get_example_data
# You may define your own write_dir and dir_name.
osiris::get_example_data(
  write_dir = getwd(),
  dir_name = "Osiris_Data_Test",
  data_link = "https://zenodo.org/record/7530067/files/Osiris_Data_Test.zip?download=1"
) -> data_folder; data_folder;

# Change data folder
# Run Step 0 to get the path to the data files
# data_folder = ADD_PATH_TO_YOUR_OSIRIS_DATA

# Test for wrf_to_osiris
osiris::wrf_to_osiris(
  wrf_ncdf = ADD_PATH_TO_YOUR_WRF_DATA,
  write_dir = paste0(data_folder,"/wrf_to_osiris"),
  osiris_ncdf = paste0(data_folder,"/climate_data/tas_Amon_CanESM5_historical_r1i1p1f1_orig.nc"),
  time_step = "3 hours",
  scenario = "historical"
)


# Test for Step 1: calculate_deltas_from_climate
osiris::calculate_deltas_from_climate(
  climate_dir = paste0(data_folder,"/climate_data"),
  write_dir = paste0(data_folder,"/outputs_calculate_delta_from_climate"),
  monthly_growing_season = paste0(data_folder,"/growing_seasons/pmonth_gslength_unifWheat_smallareamask.csv"),
  monthly_harvest_season = paste0(data_folder,"/growing_seasons/p_h_months_unifWheat_smallareamask.csv"),
  growing_season_dir = paste0(data_folder,"/outputs_growing_season_climate_data"),
  esm_name = "CanESM5",
  crops = c("Corn", "Wheat", "Rice", "Soy"),
  irrigation_rainfed = c("IRR", "RFD"),
  minlat = -87.8638,
  minlon = -179.75,
  rollingAvgYears = 1,
  tas_historical = "tas_Amon_CanESM5_historical_r1i1p1f1_gn_201001-201501.nc",
  tas_projected = "tas_Amon_CanESM5_ssp245_r1i1p1f1_gn_201501-202101.nc",
  pr_historical = "pr_Amon_CanESM5_historical_r1i1p1f1_gn_201001-201501.nc",
  pr_projected = "pr_Amon_CanESM5_ssp245_r1i1p1f1_gn_201501-202101.nc",
  historical_start_year = 2010,
  projection_start_year = 2015
)


# Test for Step 2: grid_to_basin_yield
osiris::grid_to_basin_yield(
  carbon = paste0(data_folder,"/yield_response_inputs/magicc_rcp8p5_co2.csv"),
  weight_floor_ha = 1,
  emulator_dir = paste0(data_folder,"/yield_response_fcns/ggcmi_phase2"),
  input_dir = paste0(data_folder,"/outputs_calculate_delta_from_climate"),
  area_dir = paste0(data_folder,"/area_data"),
  basin_grid = paste0(data_folder,"/mapping_data/gridData.csv"),
  basin_id = paste0(data_folder,"/mapping_data/gcam_basin_ids.csv"),
  write_dir = paste0(data_folder,"/outputs_grid_to_basin_yield"),
  wheat_area = paste0(data_folder,"/winter_and_spring_wheat_areas_v1_180627.nc4"),
  crops = c("maize", "rice", "soy", "wheat"),
  esm_name = "CanESM5",
  cm_name = "LPJmL",
  scn_name = "ssp245",
  N = 200
)


# Test for Step 3: yield_to_gcam_basin
osiris::yield_to_gcam_basin(
  write_dir = paste0(data_folder,"/outputs_yield_to_gcam_basin"),
  emulated_basin_yield_dir = paste0(data_folder,"/outputs_grid_to_basin_yield"),
  iso_GCAM_region_mapping = paste0(data_folder,"/mapping_data/iso_GCAM_regID.csv"),
  FAO_ag_mapping = paste0(data_folder,"/gcamdata_files/FAO_ag_items_PRODSTAT_expanded_corrected.csv"),
  iso_harvest_area_mapping = paste0(data_folder,"/gcamdata_files/L100.LDS_ag_HA_ha.csv"),
  iso_GCAM_basin_mapping = paste0(data_folder,"/mapping_data/gcam_basin_ids.csv"),
  esm_name = "CanESM5",
  scn_name = "ssp245",
  max_CCImult = 2.5,
  min_CCImult = 0.01,
  weight_floor_ha = 1,
  rolling_avg_years = 1,
  maxHistYear = 2015,
  minFutYear = 2015,
  maxFutYear = 2020,
  extrapolate_to = NULL
)


# Test for Step 4: create_AgProdChange_xml
osiris::create_AgProdChange_xml(
  write_dir = paste0(data_folder,"/outputs_create_AgProdChange_xml"),
  esm_name = 'CanESM5',
  scn_name = 'ssp245',
  ssp = 'ssp5',
  ag_irr_ref = paste0(data_folder,"/reference_agprodchange/L2052.AgProdChange_irr_high.csv"),
  bio_irr_ref = paste0(data_folder,"/reference_agprodchange/L2052.AgProdChange_bio_irr_ref.csv"),
  ag_impacts = paste0(data_folder,"/outputs_yield_to_gcam_basin/ag_impacts_CanESM5_ssp245_rcp_gcm_gcm_R_GLU_C_IRR_allyears_RA3_gridcull_allyroutlier.csv"),
  bio_impacts = paste0(data_folder,"/outputs_yield_to_gcam_basin/bio_impacts_CanESM5_ssp245_rcp_gcm_gcm_R_GLU_C_IRR_allyears_RA3_gridcull_allyroutlier.csv"),
  GCAM_region_mapping = paste0(data_folder,"/mapping_data/GCAM_region_names.csv"),
  timestep = 5,
  maxHistYear = 2010,
  minFutYear = 2015,
  appliedto = "full"
)


# Test for WRF diagnostic_plots
osiris::diagnostic_plots(
  hot_ssp3_apc = paste0(data_folder,"/outputs_create_AgProdChange_xml/ag_prodchange_rcp8p5_hot_ssp3_WRF_LPJmL.csv"),
  hot_ssp5_apc = paste0(data_folder,"/outputs_create_AgProdChange_xml/ag_prodchange_rcp8p5_hot_ssp5_WRF_LPJmL.csv"),
  cold_ssp3_apc = paste0(data_folder,"/outputs_create_AgProdChange_xml/ag_prodchange_rcp8p5_cold_ssp3_WRF_LPJmL.csv"),
  cold_ssp5_apc = paste0(data_folder,"/outputs_create_AgProdChange_xml/ag_prodchange_rcp8p5_cold_ssp5_WRF_LPJmL.csv"),
  reference_apc = paste0(data_folder,"/gcamdata_files/ag_prodchange_ref_IRR_MGMT.csv"),
  agyield = paste0(data_folder,"/gcamdata_files/L101.ag_Yield_kgm2_R_C_Y_GLU.csv"),
  iso_GCAM_basin_mapping = paste0(data_folder,"/mapping_data/gcam_basin_ids.csv"),
  write_dir = paste0(data_folder,"/outputs_diagnostic_plots")
)

#' @export
predict.modal_arima <- function(object, n.ahead = 10, ...) {
  y_orig <- object$y; order <- object$order; coefs <- object$coefficients
  p <- order[1]; d <- order[2]; q <- order[3]
  y_diff <- y_orig; if (d > 0) y_diff <- diff(y_orig, differences = d); n <- length(y_diff)
  c_mu <- coefs["intercept"]
  phi <- if (p > 0) coefs[paste0("ar", 1:p)] else numeric(0)
  theta <- if (q > 0) coefs[paste0("ma", 1:q)] else numeric(0)
  mu_t <- numeric(n); eps <- numeric(n); mean_y <- mean(y_diff)
  for (t in 1:n) {
    ar_term <- 0
    if (p > 0) for (i in 1:p) ar_term <- ar_term + phi[i] * (if (t-i > 0) y_diff[t-i] else mean_y)
    ma_term <- 0
    if (q > 0) for (j in 1:q) ma_term <- ma_term + theta[j] * (if (t-j > 0) eps[t-j] else 0)
    mu_t[t] <- c_mu + ar_term + ma_term; eps[t] <- y_diff[t] - mu_t[t]
  }
  pred_diff <- numeric(n.ahead)
  for (h in 1:n.ahead) {
    ar_term <- 0
    if (p > 0) for (i in 1:p) { val <- if (h-i > 0) pred_diff[h-i] else y_diff[n+h-i]; ar_term <- ar_term + phi[i]*val }
    ma_term <- 0
    if (q > 0) for (j in 1:q) { val <- if (h-j > 0) 0 else eps[n+h-j]; ma_term <- ma_term + theta[j]*val }
    pred_diff[h] <- c_mu + ar_term + ma_term
  }
  if (d > 0) { last_y <- utils::tail(y_orig, d); pred <- stats::diffinv(pred_diff, differences = d, xi = last_y)[-(1:d)] }
  else pred <- pred_diff
  return(pred)
}

#' @importFrom forecast forecast
#' @export
forecast::forecast

# ========== Internal distribution functions ==========

# --- SKN (normal) ---
.qsn <- function(prob, mu = 0, sigma = 1, gamma = 1) {
  p_mu <- 1/(gamma^2+1); res <- numeric(length(prob))
  for (i in seq_along(prob)) {
    if (prob[i] < p_mu) z <- (1/gamma)*stats::qnorm(prob[i]*(1+gamma^2)/2)
    else { val <- (prob[i]*(1+gamma^2)+gamma^2-1)/(2*gamma^2); z <- gamma*stats::qnorm(val) }
    res[i] <- mu + sigma[min(i,length(sigma))]*z
  }; return(res)
}
.rsn <- function(n, mu = 0, sigma = 1, gamma = 1) {
  u <- stats::runif(n); p_mu <- 1/(gamma^2+1); z <- numeric(n)
  idx_l <- which(u < p_mu); idx_u <- which(u >= p_mu)
  if (length(idx_l) > 0) z[idx_l] <- (1/gamma)*stats::qnorm(u[idx_l]*(1+gamma^2)/2)
  if (length(idx_u) > 0) { val <- (u[idx_u]*(1+gamma^2)+gamma^2-1)/(2*gamma^2); z[idx_u] <- gamma*stats::qnorm(val) }
  return(mu + sigma*z)
}

# --- SKT (t) ---
.dskt <- function(x, mu = 0, sigma = 1, gamma = 1, nu = 5) {
  p_skew <- 1/(gamma^2+1); z <- (x-mu)/sigma; rho <- z*(p_skew-(z<0))
  exp(lgamma((nu+1)/2)-lgamma(nu/2)-log(sigma)-0.5*log(pi*nu))*4*p_skew*(1-p_skew)*(4*rho^2/nu+1)^(-(nu+1)/2)
}
.pskt <- function(x, mu = 0, sigma = 1, gamma = 1, nu = 5) {
  cdf <- numeric(length(x))
  for (i in seq_along(x)) {
    cdf[i] <- tryCatch(stats::integrate(function(y) .dskt(y, mu=mu, sigma=sigma, gamma=gamma, nu=nu), lower=-Inf, upper=x[i], rel.tol=1e-8)$value,
                        error=function(e) 0.5)
  }; pmax(0, pmin(1, cdf))
}
.qskt <- function(prob, mu = 0, sigma = 1, gamma = 1, nu = 5) {
  res <- numeric(length(prob))
  for (i in seq_along(prob)) {
    sig_i <- sigma[min(i,length(sigma))]
    f_root <- function(x) .pskt(x, mu=mu, sigma=sig_i, gamma=gamma, nu=nu) - prob[i]
    res[i] <- tryCatch(stats::uniroot(f_root, c(mu-10*sig_i, mu+10*sig_i), tol=1e-8)$root,
                        error=function(e) tryCatch(stats::uniroot(f_root, c(mu-50*sig_i, mu+50*sig_i), tol=1e-8)$root, error=function(e2) mu))
  }; return(res)
}
.rskt <- function(n, mu = 0, sigma = 1, gamma = 1, nu = 5) {
  u_vals <- stats::rgamma(n, shape=nu/2, rate=nu/2)
  mu + sigma*(1/sqrt(u_vals))*.rsn(n, mu=0, sigma=1, gamma=gamma)
}

# --- SKL (laplace) ---
# CDF of Skewed Laplace: F(y) = p*(1-exp(-2*(1-p)*(mu-y)/sigma)) for y <= mu
#                                1-(1-p)*exp(-2*p*(y-mu)/sigma) for y > mu
.dskl <- function(x, mu = 0, sigma = 1, gamma = 1) {
  p_skew <- 1/(gamma^2+1); z <- (x-mu)/sigma; rho <- z*(p_skew-(z<0))
  2*p_skew*(1-p_skew)/sigma*exp(-2*rho)
}
.pskl <- function(x, mu = 0, sigma = 1, gamma = 1) {
  p_skew <- 1/(gamma^2+1); cdf <- numeric(length(x))
  mu_v <- rep_len(mu, length(x)); sig_v <- rep_len(sigma, length(x))
  idx_le <- which(x <= mu_v); idx_gt <- which(x > mu_v)
  if (length(idx_le) > 0) cdf[idx_le] <- p_skew*(1-exp(-2*(1-p_skew)*(mu_v[idx_le]-x[idx_le])/sig_v[idx_le]))
  if (length(idx_gt) > 0) cdf[idx_gt] <- 1-(1-p_skew)*exp(-2*p_skew*(x[idx_gt]-mu_v[idx_gt])/sig_v[idx_gt])
  pmax(0, pmin(1, cdf))
}
.qskl <- function(prob, mu = 0, sigma = 1, gamma = 1) {
  p_skew <- 1/(gamma^2+1); res <- numeric(length(prob))
  for (i in seq_along(prob)) {
    sig_i <- sigma[min(i,length(sigma))]
    if (prob[i] <= p_skew) {
      # Invert: prob = p*(1-exp(-2*(1-p)*(mu-x)/sigma))
      res[i] <- mu + sig_i*log(1-prob[i]/p_skew)/(2*(1-p_skew))
    } else {
      # Invert: prob = 1-(1-p)*exp(-2*p*(x-mu)/sigma)
      res[i] <- mu - sig_i*log((1-prob[i])/(1-p_skew))/(2*p_skew)
    }
  }; return(res)
}
.rskl <- function(n, mu = 0, sigma = 1, gamma = 1) {
  # Stochastic representation: Y = mu + sigma*sqrt(U)*Z where U ~ Exp(2), Z ~ SKN(0,1,p)
  u_vals <- stats::rexp(n, rate = 2)
  mu + sigma*sqrt(u_vals)*.rsn(n, mu=0, sigma=1, gamma=gamma)
}

#' Forecast methodology for Modal ARIMA
#'
#' @param object A modal_arima object.
#' @param h The forecast horizon.
#' @param level Confidence level for prediction intervals.
#' @param interval Method for computing prediction intervals ("asymptotic" or "bootstrap").
#' @param npaths Number of simulated paths for bootstrap intervals. Defaults to 1000.
#' @param ... Additional arguments.
#' @return A forecast object.
#' @export
#'
#' @examples
#' library(forecast)
#' 
#' # 1. Load Empirical Data (Lynx)
#' data(lynx)
#' y <- log10(lynx)
#'
#' # 2. Find the best SKD Error Distribution (Normal vs T vs Laplace) 
#' fit_n <- fit_modal_arima(y, order = c(2, 0, 0), dist = "normal")
#' fit_t <- fit_modal_arima(y, order = c(2, 0, 0), dist = "t")
#' fit_l <- fit_modal_arima(y, order = c(2, 0, 0), dist = "laplace")
#' c(Normal = AIC(fit_n), Student = AIC(fit_t), Laplace = AIC(fit_l))
#'
#' # 3. Auto Model Selection globally on the winning distribution (Skew-Normal)
#' fit_auto <- auto.modal.arima(y, d=0, max.p=2, max.q=2, dist="normal")
#'
#' # 4. Summary & Inferences
#' summary(fit_auto)
#'
#' # 5. Run residual diagnostics and Envelopes
#' diagnostics(fit_auto)
#' envelope(fit_auto, B=10)
#'
#' # 6. Produce forecasts with multiple prediction bands (alphas)
#' pred <- forecast(fit_auto, h=5, level = c(80, 95))
#'
#' # 7. Native integration with 'forecast' ecosystem
#' autoplot(pred)    
#' accuracy(pred)
forecast.modal_arima <- function(object, h = 10, level = c(80, 95), interval = c("asymptotic", "bootstrap"), npaths = 1000, ...) {
  interval <- match.arg(interval)
  dist <- if (!is.null(object$dist)) object$dist else "normal"

  if (length(object$y) <= 50 && interval == "asymptotic")
    message("Notice: The sample size is small (n <= 50). Consider using interval = 'bootstrap'.")

  pred <- predict(object, n.ahead = h)
  coefs <- object$coefficients
  p <- object$order[1]; d <- object$order[2]; q <- object$order[3]
  ar_coef <- if (p > 0) coefs[paste0("ar", 1:p)] else numeric(0)
  ma_coef <- if (q > 0) coefs[paste0("ma", 1:q)] else numeric(0)
  sigma <- coefs["sigma"]; gamma <- coefs["gamma"]
  nu <- if (dist == "t") coefs["nu"] else NULL
  c_mu <- coefs["intercept"]

  lower <- matrix(NA, nrow=h, ncol=length(level)); upper <- matrix(NA, nrow=h, ncol=length(level))

  if (interval == "asymptotic") {
    poly_diff <- c(1)
    if (d > 0) for (i in seq_len(d)) poly_diff <- stats::convolve(poly_diff, rev(c(1,-1)), type="open")
    ar_poly <- c(1, -ar_coef)
    total_ar <- stats::convolve(ar_poly, rev(poly_diff), type="open")
    total_ar_coef <- -total_ar[-1]; total_ar_coef <- total_ar_coef[total_ar_coef != 0]
    psi_weights <- if (h > 1) c(1, stats::ARMAtoMA(ar=total_ar_coef, ma=ma_coef, lag.max=h-1)) else c(1)
    sigma_h <- sigma*sqrt(cumsum(psi_weights^2))

    for (i in seq_along(level)) {
      alpha <- 1-level[i]/100; p_lower <- alpha/2; p_upper <- 1-alpha/2
      if (dist == "normal") {
        q_low <- .qsn(rep(p_lower,h), mu=0, sigma=sigma_h, gamma=gamma)
        q_up  <- .qsn(rep(p_upper,h), mu=0, sigma=sigma_h, gamma=gamma)
      } else if (dist == "t") {
        q_low <- .qskt(rep(p_lower,h), mu=0, sigma=sigma_h, gamma=gamma, nu=nu)
        q_up  <- .qskt(rep(p_upper,h), mu=0, sigma=sigma_h, gamma=gamma, nu=nu)
      } else if (dist == "laplace") {
        q_low <- .qskl(rep(p_lower,h), mu=0, sigma=sigma_h, gamma=gamma)
        q_up  <- .qskl(rep(p_upper,h), mu=0, sigma=sigma_h, gamma=gamma)
      }
      lower[,i] <- pred + q_low; upper[,i] <- pred + q_up
    }
  } else {
    y_orig <- object$y; y_diff <- y_orig
    if (d > 0) y_diff <- diff(y_orig, differences=d); n_hist <- length(y_diff)
    mean_y <- mean(y_diff); eps_hist <- numeric(n_hist)
    for (t in 1:n_hist) {
      ar_term <- 0; if (p > 0) for (i in seq_len(p)) ar_term <- ar_term+ar_coef[i]*(if (t-i>0) y_diff[t-i] else mean_y)
      ma_term <- 0; if (q > 0) for (j in seq_len(q)) ma_term <- ma_term+ma_coef[j]*(if (t-j>0) eps_hist[t-j] else 0)
      eps_hist[t] <- y_diff[t]-(c_mu+ar_term+ma_term)
    }
    sim_paths <- matrix(NA, nrow=h, ncol=npaths)
    for (b in 1:npaths) {
      yt_sim <- numeric(h)
      eps_sim <- if (dist == "normal") .rsn(h, mu=0, sigma=sigma, gamma=gamma)
                 else if (dist == "t") .rskt(h, mu=0, sigma=sigma, gamma=gamma, nu=nu)
                 else .rskl(h, mu=0, sigma=sigma, gamma=gamma)
      for (t in 1:h) {
        ar_term <- 0; if (p > 0) for (i in 1:p) { val <- if (t-i>0) yt_sim[t-i] else y_diff[n_hist+t-i]; ar_term <- ar_term+ar_coef[i]*val }
        ma_term <- 0; if (q > 0) for (j in 1:q) { val <- if (t-j>0) eps_sim[t-j] else eps_hist[n_hist+t-j]; ma_term <- ma_term+ma_coef[j]*val }
        yt_sim[t] <- c_mu+ar_term+ma_term+eps_sim[t]
      }
      if (d > 0) { last_y <- utils::tail(y_orig,d); path <- stats::diffinv(yt_sim, differences=d, xi=last_y)[-(1:d)] }
      else path <- yt_sim
      sim_paths[,b] <- path
    }
    for (i in seq_along(level)) {
      alpha <- 1-level[i]/100
      lower[,i] <- apply(sim_paths, 1, stats::quantile, probs=alpha/2)
      upper[,i] <- apply(sim_paths, 1, stats::quantile, probs=1-alpha/2)
    }
  }

  dist_label <- switch(dist, "normal"="Skew-Normal", "t"="Skewed Student-t", "laplace"="Skewed Laplace")
  y_ts <- if(stats::is.ts(object$y)) object$y else stats::ts(object$y)
  tsp_y <- stats::tsp(y_ts)
  if (is.null(tsp_y)) tsp_y <- c(1, length(y_ts), 1)
  
  mean_ts <- stats::ts(pred, start = tsp_y[2] + 1/tsp_y[3], frequency = tsp_y[3])
  lower_ts <- stats::ts(lower, start = tsp_y[2] + 1/tsp_y[3], frequency = tsp_y[3])
  upper_ts <- stats::ts(upper, start = tsp_y[2] + 1/tsp_y[3], frequency = tsp_y[3])
  fitted_ts <- stats::ts(object$fitted.values, start = tsp_y[1], frequency = tsp_y[3])
  resid_ts <- stats::ts(object$residuals, start = tsp_y[1], frequency = tsp_y[3])
  
  res <- list(mean=mean_ts, lower=lower_ts, upper=upper_ts, level=level,
              method=paste0("Modal ARIMA(", p, ",", d, ",", q, ") [", dist_label, "]"),
              x=y_ts, model=object, fitted=fitted_ts, residuals=resid_ts)
  class(res) <- "forecast"; return(res)
}

#' Diagnostic Plots for Modal ARIMA Models
#'
#' @description
#' Provides visual and statistical diagnostics for the residuals of a fitted modal ARIMA model.
#' Produces a comprehensive diagnostic panel including time series plot with
#' fitted modes, ACF/PACF of residuals, QQ-plot for normality, histogram of
#' residuals, and Ljung-Box p-values, all implemented using ggplot2.
#'
#' @param object An object of class \code{modal_arima}.
#' @param ... Additional arguments (unused).
#' @return A list of ggplot objects (invisibly) and draws the panel.
#' @import ggplot2
#' @importFrom stats acf pacf shapiro.test Box.test qqnorm qqline predict is.ts ts start frequency dnorm qnorm pnorm time residuals rgamma integrate uniroot
#' @importFrom scales comma_format
#' @importFrom grid textGrob gpar
#' @importFrom gridExtra grid.arrange
#' @export
#'
#' @examples
#' library(forecast)
#' 
#' # 1. Load Empirical Data (Lynx)
#' data(lynx)
#' y <- log10(lynx)
#'
#' # 2. Find the best SKD Error Distribution (Normal vs T vs Laplace) 
#' fit_n <- fit_modal_arima(y, order = c(2, 0, 0), dist = "normal")
#' fit_t <- fit_modal_arima(y, order = c(2, 0, 0), dist = "t")
#' fit_l <- fit_modal_arima(y, order = c(2, 0, 0), dist = "laplace")
#' c(Normal = AIC(fit_n), Student = AIC(fit_t), Laplace = AIC(fit_l))
#'
#' # 3. Auto Model Selection globally on the winning distribution (Skew-Normal)
#' fit_auto <- auto.modal.arima(y, d=0, max.p=2, max.q=2, dist="normal")
#'
#' # 4. Summary & Inferences
#' summary(fit_auto)
#'
#' # 5. Run residual diagnostics and Envelopes
#' diagnostics(fit_auto)
#' envelope(fit_auto, B=10)
#'
#' # 6. Produce forecasts with multiple prediction bands (alphas)
#' pred <- forecast(fit_auto, h=5, level = c(80, 95))
#'
#' # 7. Native integration with 'forecast' ecosystem
#' autoplot(pred)    
#' accuracy(pred)
diagnostics <- function(object, ...) {
  UseMethod("diagnostics")
}

#' @rdname diagnostics
#' @export
diagnostics.modal_arima <- function(object, ...) {
  # Global variable hack for ggplot2
  Time <- Observed <- Fitted <- Lag <- ACF <- PACF <- Residuals <- Pval <- density <- NULL

  y_orig <- object$y
  order <- object$order
  coefs <- object$coefficients
  dist <- if (!is.null(object$dist)) object$dist else "normal"

  p <- order[1]
  d <- order[2]
  q <- order[3]

  y_diff <- y_orig
  if (d > 0) y_diff <- diff(y_orig, differences = d)
  n <- length(y_diff)

  c_mu <- coefs["intercept"]
  phi <- if (p > 0) coefs[paste0("ar", 1:p)] else numeric(0)
  theta <- if (q > 0) coefs[paste0("ma", 1:q)] else numeric(0)
  sigma_hat <- coefs["sigma"]
  gamma_hat <- coefs["gamma"]
  nu_hat <- if (dist == "t") coefs["nu"] else NULL

  # Reconstruct fitted modes and residuals
  mu_t <- numeric(n); eps <- numeric(n); mean_y <- mean(y_diff)
  for (t in 1:n) {
    ar_term <- 0; if (p>0) for (i in 1:p) ar_term <- ar_term+phi[i]*(if (t-i>0) y_diff[t-i] else mean_y)
    ma_term <- 0; if (q>0) for (j in 1:q) ma_term <- ma_term+theta[j]*(if (t-j>0) eps[t-j] else 0)
    mu_t[t] <- c_mu+ar_term+ma_term; eps[t] <- y_diff[t]-mu_t[t]
  }

  # Randomized Quantile Residuals via CDF
  if (dist == "normal") {
    u <- p_skewnorm(y_diff, mu_t, sigma_hat, gamma_hat)
  } else if (dist == "t") {
    u <- numeric(n); for (i in 1:n) u[i] <- .pskt(y_diff[i], mu=mu_t[i], sigma=sigma_hat, gamma=gamma_hat, nu=nu_hat)
  } else if (dist == "laplace") {
    u <- .pskl(y_diff, mu=mu_t, sigma=sigma_hat, gamma=gamma_hat)
  }
  u <- pmax(1e-7, pmin(1-1e-7, u)); rqr <- stats::qnorm(u)

  time_idx <- if(stats::is.ts(y_diff)) as.numeric(stats::time(y_diff)) else 1:n
  df_fits <- data.frame(Time=time_idx, Observed=as.numeric(y_diff), Fitted=as.numeric(mu_t))

  p1 <- ggplot(df_fits, aes(x=Time)) + geom_line(aes(y=Observed, color="Observed"), alpha=0.6) +
    geom_line(aes(y=Fitted, color="Fitted Mode"), linewidth=0.75) +
    scale_color_manual(values=c("Observed"="gray40", "Fitted Mode"="blue")) +
    labs(title="Series & Fitted Modes", y="Value", x="Time", color=NULL) + theme_minimal(base_size=7) + theme(legend.position="top")

  bacf <- stats::acf(rqr, plot=FALSE, lag.max=20)
  df_acf <- data.frame(Lag=as.numeric(bacf$lag[-1]), ACF=as.numeric(bacf$acf[-1]))
  p2 <- ggplot(df_acf, aes(x=Lag, y=ACF)) + geom_bar(stat="identity", width=0.1, fill="blue") +
    geom_hline(yintercept=c(-1.96/sqrt(n), 1.96/sqrt(n)), linetype="dashed", color="red") + labs(title="ACF of Residuals (RQR)") + theme_minimal(base_size=7)

  bpacf <- stats::pacf(rqr, plot=FALSE, lag.max=20)
  df_pacf <- data.frame(Lag=as.numeric(bpacf$lag), PACF=as.numeric(bpacf$acf))
  p3 <- ggplot(df_pacf, aes(x=Lag, y=PACF)) + geom_bar(stat="identity", width=0.1, fill="blue") +
    geom_hline(yintercept=c(-1.96/sqrt(n), 1.96/sqrt(n)), linetype="dashed", color="red") + labs(title="PACF of Residuals (RQR)") + theme_minimal(base_size=7)

  df_rqr <- data.frame(Residuals=rqr)
  p4 <- ggplot(df_rqr, aes(sample=Residuals)) + stat_qq(color="steelblue", alpha=0.6) + stat_qq_line(color="red", linewidth=0.8) +
    labs(title="Normal QQ-Plot (RQR)") + theme_minimal(base_size=7)

  p5 <- ggplot(df_rqr, aes(x=Residuals)) + geom_histogram(aes(y=after_stat(density)), bins=20, fill="steelblue", alpha=0.4, color="white") +
    stat_function(fun=stats::dnorm, color="red", linewidth=0.8) + labs(title="Histogram (RQR)") + theme_minimal(base_size=7)

  max_lag <- min(20, n-1)
  lb_pvals <- sapply(1:max_lag, function(lag) stats::Box.test(rqr, lag=lag, type="Ljung-Box")$p.value)
  df_lb <- data.frame(Lag=1:max_lag, Pval=lb_pvals)
  p6 <- ggplot(df_lb, aes(x=Lag, y=Pval)) + geom_point(color="blue", size=2) +
    geom_hline(yintercept=0.05, linetype="dashed", color="red", linewidth=0.8) + ylim(0,1) + labs(title="Ljung-Box p-values", y="p-value") + theme_minimal(base_size=7)

  dist_label <- switch(dist, "normal"="[Normal]", "t"="[Student-t]", "laplace"="[Laplace]")
  title_grob <- grid::textGrob(paste("Modal ARIMA", dist_label, "Diagnostic Panel"), gp=grid::gpar(fontsize=10, fontface="bold"))
  gridExtra::grid.arrange(p1, p2, p3, p4, p5, p6, ncol=3, top=title_grob)

  cat("\n=== Diagnostic Tests ===\n")
  cat(sprintf("Distribution: %s\n", dist))
  sw <- stats::shapiro.test(rqr)
  cat(sprintf("Shapiro-Wilk Normality Test: W = %.4f, p-value = %.4f\n", sw$statistic, sw$p.value))
  if (sw$p.value > 0.05) cat("  -> Normality assumption met (Expected/Good fit).\n") else cat("  -> Normality assumption rejected (Requires investigation).\n")
  
  lb <- stats::Box.test(rqr, lag = 10, type = "Ljung-Box")
  cat(sprintf("Ljung-Box Test (lag=10):     X2 = %.4f, p-value = %.4f\n", lb$statistic, lb$p.value))
  if (lb$p.value > 0.05) cat("  -> Residuals are independent / No Autocorrelation (Expected/Good fit).\n") else cat("  -> Residuals are autocorrelated (Investigate lags).\n")
  cat(sprintf("\nEstimated gamma (skewness):  %.4f\n", gamma_hat))
  gamma_se <- tryCatch(sqrt(diag(solve(object$hessian))[length(object$coefficients) - if(dist=="t") 1 else 0]), error=function(e) NA)
  if (!is.na(gamma_se)) {
      p_gamma <- 2 * (1 - stats::pnorm(abs((gamma_hat - 1) / gamma_se)))
      if (p_gamma > 0.05) cat("  -> Symmetry confirmed (gamma = 1) (Expected if true series is symmetric).\n\n") else cat("  -> Symmetry rejected / SKD Skewness present (Expected by model).\n\n")
  }
  if (dist == "t") cat(sprintf("Estimated nu (d.f.):        %.4f\n\n", nu_hat))

  invisible(list(p1, p2, p3, p4, p5, p6))
}

p_skewnorm <- function(y, mu, sigma, gamma) {
  z <- (y-mu)/sigma; p_thr <- 1/(gamma^2+1); cdf <- numeric(length(y))
  idx_lt <- which(y < mu); if (length(idx_lt) > 0) cdf[idx_lt] <- 2*p_thr*stats::pnorm(z[idx_lt]*gamma)
  idx_ge <- which(y >= mu); if (length(idx_ge) > 0) cdf[idx_ge] <- p_thr+2*(1-p_thr)*(stats::pnorm(z[idx_ge]/gamma)-0.5)
  return(cdf)
}

#' Simulation Envelope Diagnostics for Modal ARIMA Models
#'
#' Constructs simulation envelopes based on the theoretical distance
#' distributions from the SKD family (Galarza et al., 2017).
#'
#' @param object An object of class \code{modal_arima}.
#' @param B Number of Monte Carlo replications for envelope construction. Default is 100.
#' @param ... Additional arguments (unused).
#' @return A ggplot object (invisibly) and draws the envelope plot.
#' @import ggplot2
#' @export
envelope <- function(object, ...) { UseMethod("envelope") }

#' @rdname envelope
#' @export
envelope.modal_arima <- function(object, B = 100, ...) {
  Theoretical <- Observed <- Lower <- Upper <- Median <- NULL # ggplot2 hack
  coefs <- object$coefficients; dist <- if (!is.null(object$dist)) object$dist else "normal"
  y_orig <- object$y; order <- object$order; p <- order[1]; d <- order[2]; q <- order[3]
  y_diff <- y_orig; if (d > 0) y_diff <- diff(y_orig, differences = d); n <- length(y_diff)
  c_mu <- coefs["intercept"]; phi <- if (p > 0) coefs[paste0("ar", 1:p)] else numeric(0); theta <- if (q > 0) coefs[paste0("ma", 1:q)] else numeric(0)
  sigma_hat <- coefs["sigma"]; gamma_hat <- coefs["gamma"]; nu_hat <- if (dist == "t") coefs["nu"] else NULL
  p_skew <- 1/(gamma_hat^2+1)

  mu_t <- numeric(n); eps <- numeric(n); mean_y <- mean(y_diff)
  for (t in 1:n) {
    ar_term <- 0; if (p>0) for (i in 1:p) ar_term <- ar_term+phi[i]*(if (t-i>0) y_diff[t-i] else mean_y)
    ma_term <- 0; if (q>0) for (j in 1:q) ma_term <- ma_term+theta[j]*(if (t-j>0) eps[t-j] else 0)
    mu_t[t] <- c_mu+ar_term+ma_term; eps[t] <- y_diff[t]-mu_t[t]
  }

  z <- eps/sigma_hat; d_obs <- z*(p_skew-(z<0)); d_obs_sorted <- sort(d_obs)

  envelope_mat <- matrix(NA, nrow=n, ncol=B)
  for (b in 1:B) {
    sim_eps <- if (dist == "normal") .rsn(n, 0, sigma_hat, gamma_hat)
               else if (dist == "t") .rskt(n, 0, sigma_hat, gamma_hat, nu_hat)
               else .rskl(n, 0, sigma_hat, gamma_hat)
    z_sim <- sim_eps/sigma_hat; d_sim <- z_sim*(p_skew-(z_sim<0)); envelope_mat[,b] <- sort(d_sim)
  }

  env_lower <- apply(envelope_mat, 1, min); env_upper <- apply(envelope_mat, 1, max); env_median <- apply(envelope_mat, 1, stats::median)
  probs <- (1:n-0.5)/n
  if (dist == "normal") theo_q <- stats::qnorm((1+probs)/2)/2
  else if (dist == "t") theo_q <- stats::qt((1+probs)/2, df=nu_hat)/2
  else theo_q <- -log(1-probs)/4 # Exponential(2) distance for SKL scaled by 1/2. CDF is 1-exp(-2*(2x)). So x = -ln(1-u)/4

  df_env <- data.frame(Theoretical=theo_q, Observed=d_obs_sorted, Lower=env_lower, Upper=env_upper, Median=env_median)
  dist_label <- switch(dist, "normal"="Skew-Normal", "t"="Skewed Student-t", "laplace"="Skewed Laplace")
  pl <- ggplot(df_env, aes(x=Theoretical)) +
    geom_ribbon(aes(ymin=Lower, ymax=Upper), fill="lightblue", alpha=0.5) +
    geom_line(aes(y=Median), color="blue", linetype="dashed", linewidth=0.6) +
    geom_point(aes(y=Observed), color="black", size=1.5, alpha=0.7) +
    geom_abline(intercept=0, slope=1, color="red", linewidth=0.8) +
    labs(title=paste0("Envelope (", dist_label, ")"), x="Theoretical Quantiles", y="Observed Distances") + theme_minimal(base_size=10)

  print(pl)
  outside <- sum(d_obs_sorted < env_lower | d_obs_sorted > env_upper)
  cat(sprintf("\n=== Envelope Diagnostics [%s] ===\n", dist_label))
  cat(sprintf("Points outside envelope: %d / %d (%.1f%%)\n", outside, n, 100*outside/n))
  invisible(pl)
}

#' @export
print.modal_arima <- function(x, ...) {
  dist <- if (!is.null(x$dist)) x$dist else "normal"
  cat("\nCall:\nfit_modal_arima(order = c(", paste(x$order, collapse=", "), "), dist = \"", dist, "\")\n", sep="")
  cat("\nCoefficients:\n")
  print(x$coefficients)
  cat("\nLog-likelihood:", round(x$loglik, 2), "\n")
  invisible(x)
}

#' @export
coef.modal_arima <- function(object, ...) {
  object$coefficients
}

#' @export
fitted.modal_arima <- function(object, ...) {
  object$fitted.values
}

#' @export
residuals.modal_arima <- function(object, ...) {
  object$residuals
}

#' @export
logLik.modal_arima <- function(object, ...) {
  structure(object$loglik, df = length(object$coefficients), nobs = length(object$y), class = "logLik")
}

#' @export
AIC.modal_arima <- function(object, ..., k = 2) {
  -2 * object$loglik + k * length(object$coefficients)
}

#' @export
BIC.modal_arima <- function(object, ...) {
  -2 * object$loglik + log(length(object$y)) * length(object$coefficients)
}

#' @export
summary.modal_arima <- function(object, ...) {
  dist <- if (!is.null(object$dist)) object$dist else "normal"

  se <- suppressWarnings(tryCatch(sqrt(diag(solve(object$hessian))), error = function(e) rep(NA, length(object$coefficients))))
  z <- object$coefficients / se
  pval <- 2 * (1 - pnorm(abs(z)))

  res <- cbind(Estimate = object$coefficients, `Std. Error` = se, `z value` = z, `Pr(>|z|)` = pval)
  
  # Separate systematic parameters from SKD (distribution) parameters
  sys_names <- grep("^(intercept|ar[0-9]+|ma[0-9]+)$", rownames(res), value = TRUE)
  skd_names <- grep("^(sigma|gamma|nu)$", rownames(res), value = TRUE)
  
  sys_mat <- res[sys_names, , drop = FALSE]
  skd_mat <- res[skd_names, , drop = FALSE]

  dist_label <- switch(dist, "normal"="Skew-Normal", "t"="Skewed Student-t", "laplace"="Skewed Laplace")
  cat("\nModal ARIMA(", paste(object$order, collapse=","), ") Model [", dist_label, "]\n", sep="")
  cat("====================================================\n")
  
  cat("\nSystematic Component:\n")
  printCoefmat(sys_mat)
  
  cat("\nSKD Family Scale & Skewness:\n")
  printCoefmat(skd_mat)
  
  cat("---\n")
  cat(sprintf("Log-likelihood: %.2f   AIC: %.2f   BIC: %.2f\n", object$loglik, AIC(object), BIC(object)))
  
  status <- if(object$convergence == 0) "Success" else paste("Code", object$convergence)
  cat(sprintf("Convergence: %s\n", status))
}

#' @export
plot.modal_arima <- function(x, ...) {
  Time <- Value <- Type <- NULL # Hack for ggplot2 notes
  dist <- if (!is.null(x$dist)) x$dist else "SKN"
  
  time_vec <- if(stats::is.ts(x$y)) as.numeric(stats::time(x$y)) else 1:length(x$y)
  
  df_orig <- data.frame(Time = time_vec, Value = as.numeric(x$y), Type = "Observed")
  df_fit <- data.frame(Time = time_vec, Value = as.numeric(x$fitted.values), Type = "Fitted Mode")
  df <- rbind(df_orig, df_fit)
  
  # Remove NAs from fitted for plotting
  df <- df[!is.na(df$Value), ]
  
  p <- ggplot2::ggplot(df, ggplot2::aes(x = Time, y = Value, color = Type, linetype = Type)) +
    ggplot2::geom_line(linewidth = 0.8) +
    ggplot2::scale_color_manual(values = c("Observed" = "gray50", "Fitted Mode" = "blue3")) +
    ggplot2::scale_linetype_manual(values = c("Observed" = "solid", "Fitted Mode" = "solid")) +
    ggplot2::labs(title = paste0("Modal ARIMA(", paste(x$order, collapse=","), ") [", dist, "] Fit"),
                  x = "Time", y = "Value", color = "", linetype = "") +
    ggplot2::theme_minimal() +
    ggplot2::theme(legend.position = "top")
    
  print(p)
  invisible(p)
}

#' Fit a Parametric Modal ARIMA Model using the SKD Family
#'
#' Fits a Modal ARIMA model where the conditional mode follows an ARIMA recursion
#' and the innovations follow a member of the SKD (Skewed Distribution) family.
#' Supports the Skew-Normal, Skewed Student-t, and Skewed Laplace distributions.
#'
#' @references
#' Galarza, C. E., Lachos, V. H., Cabral, C. R. B., and Castro, L. M. (2017).
#' Robust quantile regression using a generalized class of skewed distributions.
#' Stat, 6(1), 113-130.
#'
#' @seealso \code{\link{auto.modal.arima}}
#'
#' @note GitHub repository: \url{https://github.com/chedgala/ModalForecast}
#'
#' @param y numeric vector or time series of observations.
#' @param order A specification of the non-seasonal part of the ARIMA
#'   model: the three components (p, d, q) are the AR order, the
#'   degree of differencing, and the MA order.
#' @param dist Character string specifying the error distribution from the
#'   SKD family. One of \code{"normal"} (default) for the Skew-Normal,
#'   \code{"t"} for the Skewed Student-t (adds degrees-of-freedom parameter
#'   \code{nu}), or \code{"laplace"} for the Skewed Laplace distribution.
#'
#' @return An object of class \code{modal_arima} containing:
#'   \describe{
#'     \item{\code{y}}{The original time series.}
#'     \item{\code{order}}{The ARIMA order \code{(p,d,q)}.}
#'     \item{\code{coefficients}}{Named vector of estimated parameters.}
#'     \item{\code{loglik}}{The maximized log-likelihood.}
#'     \item{\code{hessian}}{The Hessian matrix at the optimum.}
#'     \item{\code{convergence}}{Convergence code from \code{optim}.}
#'     \item{\code{dist}}{The distribution used (\code{"normal"}, \code{"t"}, or \code{"laplace"}).}
#'   }
#' @importFrom stats arima optim qnorm runif dnorm pnorm coef AIC BIC logLik printCoefmat
#' @importFrom graphics plot
#' @export
#'
#' @examples
#' library(forecast)
#' 
#' # 1. Load Empirical Data (Lynx)
#' data(lynx)
#' y <- log10(lynx)
#'
#' # 2. Find the best SKD Error Distribution (Normal vs T vs Laplace) 
#' fit_n <- fit_modal_arima(y, order = c(2, 0, 0), dist = "normal")
#' fit_t <- fit_modal_arima(y, order = c(2, 0, 0), dist = "t")
#' fit_l <- fit_modal_arima(y, order = c(2, 0, 0), dist = "laplace")
#' c(Normal = AIC(fit_n), Student = AIC(fit_t), Laplace = AIC(fit_l))
#'
#' # 3. Auto Model Selection globally on the winning distribution (Skew-Normal)
#' fit_auto <- auto.modal.arima(y, d=0, max.p=2, max.q=2, dist="normal")
#'
#' # 4. Summary & Inferences
#' summary(fit_auto)
#'
#' # 5. Run residual diagnostics and Envelopes
#' diagnostics(fit_auto)
#' envelope(fit_auto, B=10)
#'
#' # 6. Produce forecasts with multiple prediction bands (alphas)
#' pred <- forecast(fit_auto, h=5, level = c(80, 95))
#'
#' # 7. Native integration with 'forecast' ecosystem
#' autoplot(pred)    
#' accuracy(pred)
fit_modal_arima <- function(y, order = c(1, 0, 0), dist = c("normal", "t", "laplace")) {
  dist <- match.arg(dist)
  if (length(order) != 3) stop("'order' must have length 3 (p, d, q)")
  if (anyNA(y)) stop("Missing values (NAs) are not currently supported in ModalForecast.")

  p <- order[1]
  d <- order[2]
  q <- order[3]

  y_orig <- y
  if (d > 0) {
    y <- diff(y, differences = d)
  }

  n <- length(y)

  # ==== Helper: ARIMA recursion (shared) ====
  arima_recursion <- function(params, p, q, n, y) {
    c_mu <- params[1]
    phi <- if (p > 0) params[2:(p+1)] else numeric(0)
    theta <- if (q > 0) params[(p+2):(1+p+q)] else numeric(0)

    if (p > 0) {
      ar_roots <- polyroot(c(1, -phi))
      if (any(abs(ar_roots) <= 1.001)) return(NULL)
    }
    if (q > 0) {
      ma_roots <- polyroot(c(1, theta))
      if (any(abs(ma_roots) <= 1.001)) return(NULL)
    }

    mu_t <- numeric(n)
    eps <- numeric(n)
    mean_y <- mean(y)

    for (t in 1:n) {
      ar_term <- 0
      for (i in seq_len(p)) {
        if (t - i > 0) ar_term <- ar_term + phi[i] * y[t - i]
        else ar_term <- ar_term + phi[i] * mean_y
      }
      ma_term <- 0
      for (j in seq_len(q)) {
        if (t - j > 0) ma_term <- ma_term + theta[j] * eps[t - j]
      }
      mu_t[t] <- c_mu + ar_term + ma_term
      eps[t] <- y[t] - mu_t[t]
    }
    return(list(mu_t = mu_t, eps = eps, phi = phi, theta = theta))
  }

  # ========================================================================
  # NORMAL (Skew-Normal) log-likelihood
  # ========================================================================
  neg_log_lik_normal <- function(params) {
    log_sigma <- params[length(params) - 1]
    log_gamma <- params[length(params)]
    sigma <- exp(log_sigma)
    gamma <- exp(log_gamma)

    rec <- arima_recursion(params, p, q, n, y)
    if (is.null(rec)) return(1e10)

    z <- (y - rec$mu_t) / sigma
    log_f0 <- function(val) -0.5 * log(2 * pi) - 0.5 * val^2

    idx_ge <- which(y >= rec$mu_t)
    idx_lt <- which(y < rec$mu_t)

    log_lik <- numeric(n)
    if (length(idx_ge) > 0) log_lik[idx_ge] <- log(2) - log_sigma - log(gamma + 1/gamma) + log_f0(z[idx_ge] / gamma)
    if (length(idx_lt) > 0) log_lik[idx_lt] <- log(2) - log_sigma - log(gamma + 1/gamma) + log_f0(z[idx_lt] * gamma)

    ans <- -sum(log_lik)
    if (is.na(ans) || is.infinite(ans)) ans <- 1e10
    return(ans)
  }

  grad_normal <- function(params) {
    log_sigma <- params[length(params) - 1]
    log_gamma <- params[length(params)]
    sigma <- exp(log_sigma)
    gamma <- exp(log_gamma)
    num_beta <- 1 + p + q
    G <- numeric(length(params))
    mu_t <- numeric(n); eps <- numeric(n); mean_y <- mean(y)
    V <- matrix(0, nrow = n, ncol = num_beta)
    phi <- if (p > 0) params[2:(p+1)] else numeric(0)
    theta <- if (q > 0) params[(p+2):(1+p+q)] else numeric(0)

    for (t in 1:n) {
      ar_term <- 0
      if (p > 0) for (i in seq_len(p)) {
        if (t-i > 0) ar_term <- ar_term + phi[i]*y[t-i]
        else ar_term <- ar_term + phi[i]*mean_y
      }
      ma_term <- 0
      if (q > 0) for (j in seq_len(q)) {
        if (t-j > 0) ma_term <- ma_term + theta[j]*eps[t-j]
      }
      mu_t[t] <- params[1] + ar_term + ma_term
      eps[t] <- y[t] - mu_t[t]

      Z_t <- numeric(num_beta); Z_t[1] <- 1
      if (p > 0) for (i in 1:p) Z_t[1+i] <- if (t-i > 0) y[t-i] else mean_y
      if (q > 0) for (j in 1:q) Z_t[1+p+j] <- if (t-j > 0) eps[t-j] else 0
      V_ma <- numeric(num_beta)
      if (q > 0) for (j in 1:q) if (t-j > 0) V_ma <- V_ma + theta[j]*V[t-j,]
      V[t,] <- Z_t - V_ma

      e_t <- eps[t]
      if (e_t >= 0) {
        W_t <- 1/(sigma^2*gamma^2); d_log_gamma <- -(gamma^2-1)/(gamma^2+1) + W_t*e_t^2
      } else {
        W_t <- gamma^2/sigma^2; d_log_gamma <- -(gamma^2-1)/(gamma^2+1) - W_t*e_t^2
      }
      G[1:num_beta] <- G[1:num_beta] - W_t*e_t*V[t,]
      G[length(params)-1] <- G[length(params)-1] + (1 - W_t*e_t^2)
      G[length(params)] <- G[length(params)] - d_log_gamma
    }
    return(G)
  }

  # ========================================================================
  # T (Skewed Student-t) log-likelihood
  # ========================================================================
  neg_log_lik_t <- function(params) {
    log_sigma <- params[length(params) - 2]
    log_gamma <- params[length(params) - 1]
    log_nu    <- params[length(params)]
    sigma <- exp(log_sigma); gamma <- exp(log_gamma)
    nu <- min(exp(log_nu), 200) + 2

    rec <- arima_recursion(params, p, q, n, y)
    if (is.null(rec)) return(1e10)

    p_skew <- 1 / (gamma^2 + 1)
    z <- (y - rec$mu_t) / sigma
    rho <- z * (p_skew - (z < 0))

    log_lik <- lgamma((nu+1)/2) - lgamma(nu/2) - log(sigma) - 0.5*log(pi*nu) +
      log(4*p_skew*(1-p_skew)) - ((nu+1)/2)*log(4*rho^2/nu + 1)

    ans <- -sum(log_lik)
    if (is.na(ans) || is.infinite(ans)) ans <- 1e10
    return(ans)
  }

  grad_t <- function(params) {
    log_sigma <- params[length(params)-2]
    log_gamma <- params[length(params)-1]
    log_nu    <- params[length(params)]
    sigma <- exp(log_sigma); gamma <- exp(log_gamma)
    nu <- min(exp(log_nu), 200) + 2
    num_beta <- 1+p+q; G <- numeric(length(params))
    mu_t <- numeric(n); eps <- numeric(n); mean_y <- mean(y)
    V <- matrix(0, nrow=n, ncol=num_beta)
    phi <- if (p>0) params[2:(p+1)] else numeric(0)
    theta <- if (q>0) params[(p+2):(1+p+q)] else numeric(0)
    p_skew <- 1/(gamma^2+1)

    for (t in 1:n) {
      ar_term <- 0
      if (p>0) for (i in seq_len(p)) {
        if (t-i>0) ar_term <- ar_term+phi[i]*y[t-i] else ar_term <- ar_term+phi[i]*mean_y
      }
      ma_term <- 0
      if (q>0) for (j in seq_len(q)) if (t-j>0) ma_term <- ma_term+theta[j]*eps[t-j]
      mu_t[t] <- params[1]+ar_term+ma_term; eps[t] <- y[t]-mu_t[t]

      Z_t <- numeric(num_beta); Z_t[1] <- 1
      if (p>0) for (i in 1:p) Z_t[1+i] <- if (t-i>0) y[t-i] else mean_y
      if (q>0) for (j in 1:q) Z_t[1+p+j] <- if (t-j>0) eps[t-j] else 0
      V_ma <- numeric(num_beta)
      if (q>0) for (j in 1:q) if (t-j>0) V_ma <- V_ma+theta[j]*V[t-j,]
      V[t,] <- Z_t-V_ma

      e_t <- eps[t]; z_t <- e_t/sigma
      xi_t <- if (e_t>=0) p_skew else (1-p_skew)
      rho_t <- z_t*(p_skew-(z_t<0)); denom <- nu+4*rho_t^2
      w_t <- (nu+1)/denom

      if (e_t>=0) dmu <- -w_t*8*p_skew^2*z_t/(sigma*nu)
      else dmu <- -w_t*8*(1-p_skew)^2*z_t/(sigma*nu)
      G[1:num_beta] <- G[1:num_beta]+dmu*V[t,]
      G[length(params)-2] <- G[length(params)-2]+(1-w_t*8*xi_t^2*z_t^2/nu)

      dp_dg <- -2*gamma/(gamma^2+1)^2
      d_log_p <- dp_dg/p_skew+(-dp_dg)/(1-p_skew)
      drho_dp <- z_t
      d_kernel <- -w_t*8*rho_t*drho_dp*dp_dg/nu
      G[length(params)-1] <- G[length(params)-1]-(d_log_p*gamma+d_kernel*gamma)

      dnu <- exp(log_nu)
      d_lgamma_1 <- 0.5*digamma((nu+1)/2)-0.5*digamma(nu/2) - 0.5/nu
      d_kernel_nu <- -0.5*log(4*rho_t^2/nu+1)+(nu+1)*4*rho_t^2/(2*nu^2*(4*rho_t^2/nu+1))
      G[length(params)] <- G[length(params)]-(d_lgamma_1+d_kernel_nu)*dnu
    }
    return(G)
  }

  # ========================================================================
  # LAPLACE (Skewed Laplace) log-likelihood
  # f(y|mu,sigma,p) = 2*p*(1-p)/sigma * exp(-2*rho_p((y-mu)/sigma))
  # ========================================================================
  neg_log_lik_laplace <- function(params) {
    log_sigma <- params[length(params) - 1]
    log_gamma <- params[length(params)]
    sigma <- exp(log_sigma); gamma <- exp(log_gamma)

    rec <- arima_recursion(params, p, q, n, y)
    if (is.null(rec)) return(1e10)

    p_skew <- 1 / (gamma^2 + 1)
    z <- (y - rec$mu_t) / sigma
    rho <- z * (p_skew - (z < 0))

    log_lik <- log(2*p_skew*(1-p_skew)) - log(sigma) - 2*rho

    ans <- -sum(log_lik)
    if (is.na(ans) || is.infinite(ans)) ans <- 1e10
    return(ans)
  }

  grad_laplace <- function(params) {
    log_sigma <- params[length(params)-1]
    log_gamma <- params[length(params)]
    sigma <- exp(log_sigma); gamma <- exp(log_gamma)
    num_beta <- 1+p+q; G <- numeric(length(params))
    mu_t <- numeric(n); eps <- numeric(n); mean_y <- mean(y)
    V <- matrix(0, nrow=n, ncol=num_beta)
    phi <- if (p>0) params[2:(p+1)] else numeric(0)
    theta <- if (q>0) params[(p+2):(1+p+q)] else numeric(0)
    p_skew <- 1/(gamma^2+1)

    for (t in 1:n) {
      ar_term <- 0
      if (p>0) for (i in seq_len(p)) {
        if (t-i>0) ar_term <- ar_term+phi[i]*y[t-i] else ar_term <- ar_term+phi[i]*mean_y
      }
      ma_term <- 0
      if (q>0) for (j in seq_len(q)) if (t-j>0) ma_term <- ma_term+theta[j]*eps[t-j]
      mu_t[t] <- params[1]+ar_term+ma_term; eps[t] <- y[t]-mu_t[t]

      Z_t <- numeric(num_beta); Z_t[1] <- 1
      if (p>0) for (i in 1:p) Z_t[1+i] <- if (t-i>0) y[t-i] else mean_y
      if (q>0) for (j in 1:q) Z_t[1+p+j] <- if (t-j>0) eps[t-j] else 0
      V_ma <- numeric(num_beta)
      if (q>0) for (j in 1:q) if (t-j>0) V_ma <- V_ma+theta[j]*V[t-j,]
      V[t,] <- Z_t-V_ma

      e_t <- eps[t]; z_t <- e_t/sigma
      # d/d_mu of -2*rho_p(z) where rho_p(z) = z*(p - I(z<0))
      # d(rho)/d(mu) = -(1/sigma)*(p - I(z<0))
      sign_contrib <- if (e_t >= 0) p_skew else -(1-p_skew)
      # grad w.r.t. beta: increases neg_log_lik
      G[1:num_beta] <- G[1:num_beta] - (2/sigma)*sign_contrib*V[t,]

      # grad w.r.t. log_sigma: d/d_log_sigma = 1 - 2*|rho_p(z)| (chain rule)
      rho_t <- z_t*(p_skew-(z_t<0))
      G[length(params)-1] <- G[length(params)-1] + (1 - 2*rho_t)

      # grad w.r.t. log_gamma
      dp_dg <- -2*gamma/(gamma^2+1)^2
      d_log_p <- dp_dg/p_skew + (-dp_dg)/(1-p_skew)
      drho_dp <- z_t
      G[length(params)] <- G[length(params)] - (d_log_p + 2*drho_dp*dp_dg)*gamma
    }
    return(G)
  }

  # ========================================================================
  # Initial values based on standard ARIMA
  # ========================================================================
  init_fit <- suppressWarnings(stats::arima(y, order = c(p, 0, q), method = "CSS"))
  init_c <- if ("intercept" %in% names(init_fit$coef)) init_fit$coef["intercept"] else 0
  init_phi <- if (p > 0) init_fit$coef[paste0("ar", 1:p)] else numeric(0)
  init_theta <- if (q > 0) init_fit$coef[paste0("ma", 1:q)] else numeric(0)
  init_sigma <- sqrt(init_fit$sigma2)
  if (init_sigma < 1e-4) init_sigma <- 1

  if (dist == "normal") {
    init_params <- c(init_c, init_phi, init_theta, log(init_sigma), 0)
    opt <- optim(par = init_params, fn = neg_log_lik_normal, gr = grad_normal,
                 method = "BFGS", hessian = TRUE)
    est_sigma <- exp(opt$par[length(opt$par)-1])
    est_gamma <- exp(opt$par[length(opt$par)])
    coef_names <- c("intercept")
    if (p > 0) coef_names <- c(coef_names, paste0("ar", 1:p))
    if (q > 0) coef_names <- c(coef_names, paste0("ma", 1:q))
    coef_names <- c(coef_names, "sigma", "gamma")
    coefficients <- c(opt$par[1],
                      if (p>0) opt$par[2:(p+1)] else NULL,
                      if (q>0) opt$par[(p+2):(1+p+q)] else NULL,
                      est_sigma, est_gamma)
    names(coefficients) <- coef_names

  } else if (dist == "t") {
    init_params <- c(init_c, init_phi, init_theta, log(init_sigma), 0, log(3))
    opt <- optim(par = init_params, fn = neg_log_lik_t, gr = grad_t,
                 method = "BFGS", hessian = TRUE)
    est_sigma <- exp(opt$par[length(opt$par)-2])
    est_gamma <- exp(opt$par[length(opt$par)-1])
    est_nu    <- min(exp(opt$par[length(opt$par)]), 200) + 2
    coef_names <- c("intercept")
    if (p > 0) coef_names <- c(coef_names, paste0("ar", 1:p))
    if (q > 0) coef_names <- c(coef_names, paste0("ma", 1:q))
    coef_names <- c(coef_names, "sigma", "gamma", "nu")
    coefficients <- c(opt$par[1],
                      if (p>0) opt$par[2:(p+1)] else NULL,
                      if (q>0) opt$par[(p+2):(1+p+q)] else NULL,
                      est_sigma, est_gamma, est_nu)
    names(coefficients) <- coef_names

  } else if (dist == "laplace") {
    init_params <- c(init_c, init_phi, init_theta, log(init_sigma), 0)
    opt <- optim(par = init_params, fn = neg_log_lik_laplace, gr = grad_laplace,
                 method = "BFGS", hessian = TRUE)
    est_sigma <- exp(opt$par[length(opt$par)-1])
    est_gamma <- exp(opt$par[length(opt$par)])
    coef_names <- c("intercept")
    if (p > 0) coef_names <- c(coef_names, paste0("ar", 1:p))
    if (q > 0) coef_names <- c(coef_names, paste0("ma", 1:q))
    coef_names <- c(coef_names, "sigma", "gamma")
    coefficients <- c(opt$par[1],
                      if (p>0) opt$par[2:(p+1)] else NULL,
                      if (q>0) opt$par[(p+2):(1+p+q)] else NULL,
                      est_sigma, est_gamma)
    names(coefficients) <- coef_names
  }

  rec <- arima_recursion(opt$par, p, q, n, y)
  eps_vals <- rep(NA, length(y_orig))
  fit_vals <- rep(NA, length(y_orig))
  
  if (!is.null(rec)) {
     if (d > 0) {
        eps_vals[(d+1):length(y_orig)] <- rec$eps
        fit_vals[(d+1):length(y_orig)] <- y_orig[(d+1):length(y_orig)] - rec$eps
     } else {
        eps_vals <- rec$eps
        fit_vals <- rec$mu_t
     }
  }
  if (stats::is.ts(y_orig)) {
     eps_vals <- stats::ts(eps_vals, start=stats::start(y_orig), frequency=stats::frequency(y_orig))
     fit_vals <- stats::ts(fit_vals, start=stats::start(y_orig), frequency=stats::frequency(y_orig))
  }

  out <- list(
    y = y_orig,
    order = order,
    coefficients = coefficients,
    loglik = -opt$value,
    hessian = opt$hessian,
    convergence = opt$convergence,
    dist = dist,
    fitted.values = fit_vals,
    residuals = eps_vals
  )
  class(out) <- "modal_arima"
  return(out)
}

test_that("auto.modal_arima works", {
  set.seed(123)
  y <- stats::arima.sim(n = 50, list(ar = 0.5)) + 10
  fit <- auto.modal.arima(as.numeric(y), max.p = 1, max.q = 0)
  expect_s3_class(fit, "modal_arima")
  expect_true(fit$convergence == 0)
})

test_that("predict and forecast work", {
  set.seed(123)
  y <- stats::arima.sim(n = 50, list(ar = 0.5)) + 10
  fit <- fit_modal_arima(as.numeric(y), order = c(1, 0, 0))
  
  pred <- predict(fit, n.ahead = 5)
  expect_length(pred, 5)
  
  fc <- forecast(fit, h = 5)
  expect_s3_class(fc, "forecast")
  expect_length(fc$mean, 5)
})

#' Automatic selection of Parametric Modal ARIMA model
#'
#' @details
#' This function performs a grid search over AR and MA orders to find the
#' optimal Modal ARIMA model based on the selected information criterion (AIC or BIC).
#'
#' @references
#' Galarza, C. E., Lachos, V. H., Cabral, C. R. B., and Castro, L. M. (2017).
#' Robust quantile regression using a generalized class of skewed distributions.
#' Stat, 6(1), 113-130.
#'
#' @seealso \code{\link{fit_modal_arima}}
#'
#' @note GitHub repository: \url{https://github.com/chedgala/ModalForecast}
#'
#' @param y numeric vector or time series of observations
#' @param d Integer, degree of differencing. If NA, it's determined automatically.
#' @param max.p Maximum AR order
#' @param max.q Maximum MA order
#' @param ic Information criterion to be used in model selection ("aic", "bic")
#' @param dist Character string specifying the error distribution.
#'   \code{"normal"} (default) for Skew-Normal, \code{"t"} for Skewed Student-t,
#'   \code{"laplace"} for Skewed Laplace.
#'
#' @return An object of class \code{modal_arima}.
#' @importFrom stats AIC BIC
#' @importFrom forecast ndiffs
#' @export
#'
#' @examples
#' library(forecast)
#' 
#' # 1. Load Empirical Data (Lynx)
#' data(lynx)
#' y <- log10(lynx)
#'
#' # 2. Find the best SKD Error Distribution (Normal vs T vs Laplace) 
#' fit_n <- fit_modal_arima(y, order = c(2, 0, 0), dist = "normal")
#' fit_t <- fit_modal_arima(y, order = c(2, 0, 0), dist = "t")
#' fit_l <- fit_modal_arima(y, order = c(2, 0, 0), dist = "laplace")
#' c(Normal = AIC(fit_n), Student = AIC(fit_t), Laplace = AIC(fit_l))
#'
#' # 3. Auto Model Selection globally on the winning distribution (Skew-Normal)
#' fit_auto <- auto.modal.arima(y, d=0, max.p=2, max.q=2, dist="normal")
#'
#' # 4. Summary & Inferences
#' summary(fit_auto)
#'
#' # 5. Run residual diagnostics and Envelopes
#' diagnostics(fit_auto)
#' envelope(fit_auto, B=10)
#'
#' # 6. Produce forecasts with multiple prediction bands (alphas)
#' pred <- forecast(fit_auto, h=5, level = c(80, 95))
#'
#' # 7. Native integration with 'forecast' ecosystem
#' autoplot(pred)    
#' accuracy(pred)
auto.modal.arima <- function(y, d = NA, max.p = 5, max.q = 5,
                              ic = c("aic", "bic"),
                              dist = c("normal", "t", "laplace")) {
  ic <- match.arg(ic)
  dist <- match.arg(dist)

  if (is.na(d)) {
    if (!requireNamespace("forecast", quietly = TRUE)) {
      stop("Package 'forecast' is needed to automatically determine 'd'. Please install it or specify 'd'.")
    }
    d <- forecast::ndiffs(y)
  }

  best_ic <- Inf
  best_mod <- NULL

  for (p in 0:max.p) {
    for (q in 0:max.q) {
      if (p == 0 && q == 0) next
      mod <- tryCatch({
        fit_modal_arima(y, order = c(p, d, q), dist = dist)
      }, error = function(e) NULL)

      if (!is.null(mod) && mod$convergence == 0) {
        current_ic <- if (ic == "aic") AIC(mod) else BIC(mod)
        if (current_ic < best_ic) {
          best_ic <- current_ic
          best_mod <- mod
        }
      }
    }
  }

  if (is.null(best_mod)) {
    best_mod <- fit_modal_arima(y, order = c(0, d, 0), dist = dist)
  }

  return(best_mod)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auto_modal_arima.R
\name{auto.modal.arima}
\alias{auto.modal.arima}
\title{Automatic selection of Parametric Modal ARIMA model}
\usage{
auto.modal.arima(
  y,
  d = NA,
  max.p = 5,
  max.q = 5,
  ic = c("aic", "bic"),
  dist = c("normal", "t", "laplace")
)
}
\arguments{
\item{y}{numeric vector or time series of observations}

\item{d}{Integer, degree of differencing. If NA, it's determined automatically.}

\item{max.p}{Maximum AR order}

\item{max.q}{Maximum MA order}

\item{ic}{Information criterion to be used in model selection ("aic", "bic")}

\item{dist}{Character string specifying the error distribution.
\code{"normal"} (default) for Skew-Normal, \code{"t"} for Skewed Student-t,
\code{"laplace"} for Skewed Laplace.}
}
\value{
An object of class \code{modal_arima}.
}
\description{
Automatic selection of Parametric Modal ARIMA model
}
\details{
This function performs a grid search over AR and MA orders to find the
optimal Modal ARIMA model based on the selected information criterion (AIC or BIC).
}
\note{
GitHub repository: \url{https://github.com/chedgala/ModalForecast}
}
\examples{
library(forecast)

# 1. Load Empirical Data (Lynx)
data(lynx)
y <- log10(lynx)

# 2. Find the best SKD Error Distribution (Normal vs T vs Laplace) 
fit_n <- fit_modal_arima(y, order = c(2, 0, 0), dist = "normal")
fit_t <- fit_modal_arima(y, order = c(2, 0, 0), dist = "t")
fit_l <- fit_modal_arima(y, order = c(2, 0, 0), dist = "laplace")
c(Normal = AIC(fit_n), Student = AIC(fit_t), Laplace = AIC(fit_l))

# 3. Auto Model Selection globally on the winning distribution (Skew-Normal)
fit_auto <- auto.modal.arima(y, d=0, max.p=2, max.q=2, dist="normal")

# 4. Summary & Inferences
summary(fit_auto)

# 5. Run residual diagnostics and Envelopes
diagnostics(fit_auto)
envelope(fit_auto, B=10)

# 6. Produce forecasts with multiple prediction bands (alphas)
pred <- forecast(fit_auto, h=5, level = c(80, 95))

# 7. Native integration with 'forecast' ecosystem
autoplot(pred)    
accuracy(pred)
}
\references{
Galarza, C. E., Lachos, V. H., Cabral, C. R. B., and Castro, L. M. (2017).
Robust quantile regression using a generalized class of skewed distributions.
Stat, 6(1), 113-130.
}
\seealso{
\code{\link{fit_modal_arima}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/modal_arima.R
\name{fit_modal_arima}
\alias{fit_modal_arima}
\title{Fit a Parametric Modal ARIMA Model using the SKD Family}
\usage{
fit_modal_arima(y, order = c(1, 0, 0), dist = c("normal", "t", "laplace"))
}
\arguments{
\item{y}{numeric vector or time series of observations.}

\item{order}{A specification of the non-seasonal part of the ARIMA
model: the three components (p, d, q) are the AR order, the
degree of differencing, and the MA order.}

\item{dist}{Character string specifying the error distribution from the
SKD family. One of \code{"normal"} (default) for the Skew-Normal,
\code{"t"} for the Skewed Student-t (adds degrees-of-freedom parameter
\code{nu}), or \code{"laplace"} for the Skewed Laplace distribution.}
}
\value{
An object of class \code{modal_arima} containing:
  \describe{
    \item{\code{y}}{The original time series.}
    \item{\code{order}}{The ARIMA order \code{(p,d,q)}.}
    \item{\code{coefficients}}{Named vector of estimated parameters.}
    \item{\code{loglik}}{The maximized log-likelihood.}
    \item{\code{hessian}}{The Hessian matrix at the optimum.}
    \item{\code{convergence}}{Convergence code from \code{optim}.}
    \item{\code{dist}}{The distribution used (\code{"normal"}, \code{"t"}, or \code{"laplace"}).}
  }
}
\description{
Fits a Modal ARIMA model where the conditional mode follows an ARIMA recursion
and the innovations follow a member of the SKD (Skewed Distribution) family.
Supports the Skew-Normal, Skewed Student-t, and Skewed Laplace distributions.
}
\note{
GitHub repository: \url{https://github.com/chedgala/ModalForecast}
}
\examples{
library(forecast)

# 1. Load Empirical Data (Lynx)
data(lynx)
y <- log10(lynx)

# 2. Find the best SKD Error Distribution (Normal vs T vs Laplace) 
fit_n <- fit_modal_arima(y, order = c(2, 0, 0), dist = "normal")
fit_t <- fit_modal_arima(y, order = c(2, 0, 0), dist = "t")
fit_l <- fit_modal_arima(y, order = c(2, 0, 0), dist = "laplace")
c(Normal = AIC(fit_n), Student = AIC(fit_t), Laplace = AIC(fit_l))

# 3. Auto Model Selection globally on the winning distribution (Skew-Normal)
fit_auto <- auto.modal.arima(y, d=0, max.p=2, max.q=2, dist="normal")

# 4. Summary & Inferences
summary(fit_auto)

# 5. Run residual diagnostics and Envelopes
diagnostics(fit_auto)
envelope(fit_auto, B=10)

# 6. Produce forecasts with multiple prediction bands (alphas)
pred <- forecast(fit_auto, h=5, level = c(80, 95))

# 7. Native integration with 'forecast' ecosystem
autoplot(pred)    
accuracy(pred)
}
\references{
Galarza, C. E., Lachos, V. H., Cabral, C. R. B., and Castro, L. M. (2017).
Robust quantile regression using a generalized class of skewed distributions.
Stat, 6(1), 113-130.
}
\seealso{
\code{\link{auto.modal.arima}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{envelope}
\alias{envelope}
\alias{envelope.modal_arima}
\title{Simulation Envelope Diagnostics for Modal ARIMA Models}
\usage{
envelope(object, ...)

\method{envelope}{modal_arima}(object, B = 100, ...)
}
\arguments{
\item{object}{An object of class \code{modal_arima}.}

\item{...}{Additional arguments (unused).}

\item{B}{Number of Monte Carlo replications for envelope construction. Default is 100.}
}
\value{
A ggplot object (invisibly) and draws the envelope plot.
}
\description{
Constructs simulation envelopes based on the theoretical distance
distributions from the SKD family (Galarza et al., 2017).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{diagnostics}
\alias{diagnostics}
\alias{diagnostics.modal_arima}
\title{Diagnostic Plots for Modal ARIMA Models}
\usage{
diagnostics(object, ...)

\method{diagnostics}{modal_arima}(object, ...)
}
\arguments{
\item{object}{An object of class \code{modal_arima}.}

\item{...}{Additional arguments (unused).}
}
\value{
A list of ggplot objects (invisibly) and draws the panel.
}
\description{
Provides visual and statistical diagnostics for the residuals of a fitted modal ARIMA model.
Produces a comprehensive diagnostic panel including time series plot with
fitted modes, ACF/PACF of residuals, QQ-plot for normality, histogram of
residuals, and Ljung-Box p-values, all implemented using ggplot2.
}
\examples{
library(forecast)

# 1. Load Empirical Data (Lynx)
data(lynx)
y <- log10(lynx)

# 2. Find the best SKD Error Distribution (Normal vs T vs Laplace) 
fit_n <- fit_modal_arima(y, order = c(2, 0, 0), dist = "normal")
fit_t <- fit_modal_arima(y, order = c(2, 0, 0), dist = "t")
fit_l <- fit_modal_arima(y, order = c(2, 0, 0), dist = "laplace")
c(Normal = AIC(fit_n), Student = AIC(fit_t), Laplace = AIC(fit_l))

# 3. Auto Model Selection globally on the winning distribution (Skew-Normal)
fit_auto <- auto.modal.arima(y, d=0, max.p=2, max.q=2, dist="normal")

# 4. Summary & Inferences
summary(fit_auto)

# 5. Run residual diagnostics and Envelopes
diagnostics(fit_auto)
envelope(fit_auto, B=10)

# 6. Produce forecasts with multiple prediction bands (alphas)
pred <- forecast(fit_auto, h=5, level = c(80, 95))

# 7. Native integration with 'forecast' ecosystem
autoplot(pred)    
accuracy(pred)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{forecast.modal_arima}
\alias{forecast.modal_arima}
\title{Forecast methodology for Modal ARIMA}
\usage{
\method{forecast}{modal_arima}(
  object,
  h = 10,
  level = c(80, 95),
  interval = c("asymptotic", "bootstrap"),
  npaths = 1000,
  ...
)
}
\arguments{
\item{object}{A modal_arima object.}

\item{h}{The forecast horizon.}

\item{level}{Confidence level for prediction intervals.}

\item{interval}{Method for computing prediction intervals ("asymptotic" or "bootstrap").}

\item{npaths}{Number of simulated paths for bootstrap intervals. Defaults to 1000.}

\item{...}{Additional arguments.}
}
\value{
A forecast object.
}
\description{
Forecast methodology for Modal ARIMA
}
\examples{
library(forecast)

# 1. Load Empirical Data (Lynx)
data(lynx)
y <- log10(lynx)

# 2. Find the best SKD Error Distribution (Normal vs T vs Laplace) 
fit_n <- fit_modal_arima(y, order = c(2, 0, 0), dist = "normal")
fit_t <- fit_modal_arima(y, order = c(2, 0, 0), dist = "t")
fit_l <- fit_modal_arima(y, order = c(2, 0, 0), dist = "laplace")
c(Normal = AIC(fit_n), Student = AIC(fit_t), Laplace = AIC(fit_l))

# 3. Auto Model Selection globally on the winning distribution (Skew-Normal)
fit_auto <- auto.modal.arima(y, d=0, max.p=2, max.q=2, dist="normal")

# 4. Summary & Inferences
summary(fit_auto)

# 5. Run residual diagnostics and Envelopes
diagnostics(fit_auto)
envelope(fit_auto, B=10)

# 6. Produce forecasts with multiple prediction bands (alphas)
pred <- forecast(fit_auto, h=5, level = c(80, 95))

# 7. Native integration with 'forecast' ecosystem
autoplot(pred)    
accuracy(pred)
}
